@@ -104,7 +104,7 @@ run (GVfsJob *job)
   if (class->poll_mountable == NULL)
     {
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                        _("Operation not supported by backend"));
+                        _("Operation not supported"));
       return;
     }
 
